#!/usr/bin/env python

import click
import json
import requests
import logging

ENVIRONS = {
    "https://cts.engineering.redhat.com/api/1": {
        "auth_endpoint": "https://auth.redhat.com/auth/realms/EmployeeIDP/protocol/openid-connect/auth",  # noqa
        "token_endpoint": "https://auth.redhat.com/auth/realms/EmployeeIDP/protocol/openid-connect/token",  # noqa
    },
    "https://cts.stage.engineering.redhat.com/api/1": {
        "auth_endpoint": "https://auth.stage.redhat.com/auth/realms/EmployeeIDP/protocol/openid-connect/auth",  # noqa
        "token_endpoint": "https://auth.stage.redhat.com/auth/realms/EmployeeIDP/protocol/openid-connect/token",  # noqa
    },
}


class CTS:
    def __init__(self, url):
        self.api_url = url.rstrip("/") + "/api/1"
        self.session = requests.Session()
        self.session.timeout = (5, 10)
        if self.api_url in ENVIRONS:
            from token_manager import TokenManager
            token_manager = TokenManager(
                "compose-tracking-service",
                ENVIRONS[self.api_url]["auth_endpoint"],
                ENVIRONS[self.api_url]["token_endpoint"],
            )
            self.session.auth = token_manager.auth

    def get_compose(self, compose_id):
        resp = self.session.get(f"{self.api_url}/composes/{compose_id}")
        resp.raise_for_status()
        return resp.json()

    def _edit_compose(self, compose_id, json_data):
        resp = self.session.patch(
            f"{self.api_url}/composes/{compose_id}", json=json_data
        )
        resp.raise_for_status()

    def tag_action(self, action, compose_id, tag, comment):
        self._edit_compose(
            compose_id, {"action": action, "tag": tag, "user_data": comment},
        )

    def untag(self, compose_id, tag, comment):
        print(f"Untagging {compose_id} with {tag}.")
        self.tag_action("untag", compose_id, tag, comment)

    def tag(self, compose_id, tag, comment):
        print(f"Tagging {compose_id} with {tag}.")
        self.tag_action("tag", compose_id, tag, comment)

    def create_tag(self, name, desc, doc, comment):
        resp = self.session.post(
            f"{self.api_url}/tags/",
            json={
                "name": name,
                "description": desc,
                "documentation": doc,
                "user_data": comment,
            },
        )
        resp.raise_for_status()

    def get_tag(self, tag_name):
        resp = self.session.get(f"{self.api_url}/tags/?name={tag_name}")
        resp.raise_for_status()
        return resp.json()

    def add_tagger(self, tag_id, tagger, comment):
        print(f"Adding {tagger} to taggers of tag {tag_id}")
        self.tagger_action("add_tagger", tag_id, tagger, comment)

    def add_untagger(self, tag_id, tagger, comment):
        print(f"Adding {tagger} to untaggers of tag {tag_id}")
        self.tagger_action("add_untagger", tag_id, tagger, comment)

    def remove_tagger(self, tag_id, tagger, comment):
        print(f"Removing {tagger} from taggers of tag {tag_id}")
        self.tagger_action("remove_untagger", tag_id, tagger, comment)

    def remove_untagger(self, tag_id, tagger, comment):
        print(f"Removing {tagger} from untaggers of tag {tag_id}")
        self.tagger_action("remove_untagger", tag_id, tagger, comment)

    def tagger_action(self, action, tag_id, username, comment):
        resp = self.session.patch(
            f"{self.api_url}/tags/{tag_id}",
            json={
                "action": action,
                "username": username,
                "user_data": comment,
            },
        )
        resp.raise_for_status()

    def tag_info(self, tag_name_or_id):
        try:
            return int(tag_name_or_id)
        except ValueError:
            pass
        resp = self.session.get(
            f"{self.api_url}/tags/",
            params={"name": tag_name_or_id},
        )
        resp.raise_for_status()
        data = resp.json()
        try:
            return data["items"][0]
        except IndexError:
            raise RuntimeError(f"No such tag: {tag_name_or_id}")

    def resolve_tag(self, tag_name_or_id):
        return self.tag_info(tag_name_or_id)["id"]

    def set_compose_url(self, compose_id, url, comment):
        self._edit_compose(
            compose_id,
            {"action": "set_url", "compose_url": url, "user_data": comment},
        )


@click.group()
@click.option("--server", default="https://cts.engineering.redhat.com/")
@click.option("--debug", is_flag=True)
@click.pass_context
def cli(ctx, server, debug):
    logging.basicConfig(
        format="%(asctime)s - %(levelname)s - %(message)s",
        level=logging.DEBUG if debug else logging.WARNING,
    )
    ctx.obj = CTS(server)


def tag_operation(session, func, tag, compose_id, comment, without_children):
    for parent_id in compose_id:
        func(parent_id, tag, comment)
        if not without_children:
            for child_id in session.get_compose(parent_id)["children"]:
                func(child_id, tag, comment)


@cli.command()
@click.argument("tag")
@click.argument("compose_id", nargs=-1)
@click.option("--comment", default="", help="Optional comment for the action")
@click.option(
    "--without-children",
    is_flag=True,
    help="Modify only the specified composes and not their children",
)
@click.pass_obj
def untag(session, tag, compose_id, comment, without_children):
    tag_operation(session, session.untag, tag, compose_id, comment, without_children)


@cli.command()
@click.argument("tag")
@click.argument("compose_id", nargs=-1)
@click.option("--comment", default="", help="Optional comment for the action")
@click.option(
    "--without-children",
    is_flag=True,
    help="Modify only the specified composes and not their children",
)
@click.pass_obj
def tag(session, tag, compose_id, comment, without_children):
    try:
        tag_operation(session, session.tag, tag, compose_id, comment, without_children)
    except requests.exceptions.HTTPError as exc:
        print(exc.response.json())
        raise


@cli.command()
@click.argument("compose_id")
@click.pass_obj
def info(session, compose_id):
    compose = session.get_compose(compose_id)
    print(json.dumps(compose, indent=2, sort_keys=True))


@cli.command()
@click.argument("name")
@click.argument("description")
@click.argument("documentation")
@click.option("--comment", default="", help="Optional comment for the action")
@click.pass_obj
def create_tag(session, name, description, documentation, comment):
    session.create_tag(name, description, documentation, comment)


@cli.command()
@click.argument("tag_name")
@click.argument("username")
@click.option("--comment", default="", help="Optional comment for the action")
@click.pass_obj
def add_tagger(session, tag_name, username, comment):
    session.add_tagger(session.resolve_tag(tag_name), username, comment)


@cli.command()
@click.argument("tag_name")
@click.argument("username")
@click.option("--comment", default="", help="Optional comment for the action")
@click.pass_obj
def add_untagger(session, tag_name, username, comment):
    session.add_untagger(session.resolve_tag(tag_name), username, comment)


@cli.command()
@click.argument("tag_name")
@click.argument("username")
@click.option("--comment", default="", help="Optional comment for the action")
@click.pass_obj
def remove_tagger(session, tag_name, username, comment):
    session.remove_tagger(session.resolve_tag(tag_name), username, comment)


@cli.command()
@click.argument("tag_name")
@click.argument("username")
@click.option("--comment", default="", help="Optional comment for the action")
@click.pass_obj
def remove_untagger(session, tag_name, username, comment):
    session.remove_untagger(session.resolve_tag(tag_name), username, comment)


@cli.command()
@click.argument("compose_id")
@click.argument("url")
@click.option("--comment", default="", help="Optional comment for the action")
@click.pass_obj
def set_url(session, compose_id, url, comment):
    session.set_compose_url(compose_id, url, comment)
